@@ -96,7 +96,7 @@ def convert_dbus_exceptions(fn, *args, **kwargs):
     '''Convert D-Bus exceptions to their actual exception types'''
     try:
         return fn(*args, **kwargs)
-    except dbus.DBusException, e:
+    except dbus.DBusException as e:
         if e._dbus_error_name == PermissionDeniedByPolicy._dbus_error_name:
             raise PermissionDeniedByPolicy(str(e))
         elif e._dbus_error_name == InvalidModeException._dbus_error_name:
@@ -127,10 +127,11 @@ class Backend(dbus.service.Object):
         In order to be fast and not block client side applications for very
         long, detect can be set to False; in that case this constructor does
         not detect hardware and drivers, and client-side applications must call
-        detect() at program start.
+        detect() or db_init() at program start.
         '''
         self.handler_dir = handler_dir
         self.handler_pool = {}
+        self.driver_dbs = None
         self.hardware = None
 
         # cached D-BUS interfaces for _check_polkit_privilege()
@@ -159,16 +160,41 @@ class Backend(dbus.service.Object):
         bouncing progress bar (if appropriate). If the backend is already
         initialized, this returns immediately.
 
-        This must be called once at client-side program start.
+        This must be called once at client-side program start. If the Backend
+        object is initialized with argument "detect=True", this happens
+        automatically. If the client just wants to perform a search_driver()
+        operation on remote driver DBs on already detected hardware, it can
+        only call db_init() instead.
         '''
         self._reset_timeout()
         self._check_polkit_privilege(sender, conn, 'com.ubuntu.devicedriver.info')
 
         if not self.hardware:
             self.hardware = detection.get_hardware()
+            self.db_init()
+            self._detect_handlers()
+
+    @dbus.service.method(DBUS_INTERFACE_NAME,
+        in_signature='', out_signature='', sender_keyword='sender',
+        connection_keyword='conn')
+    def db_init(self, sender=None, conn=None):
+        '''Initialize the driver databases for search_driver()
+
+        When a client wants to search a remote driver for a piece of already
+        detected hardware (for example a printer detected by CUPS) we do not
+        need the time-consuming operation of detect(). It is enough to
+        initialize the driver databases.
+
+        This must be called once at client-side program start, if not using
+        detect() or initializing the Backend with argument "detect=True".
+        '''
+        self._reset_timeout()
+        self._check_polkit_privilege(sender, conn, 'com.ubuntu.devicedriver.info')
+
+        if not self.driver_dbs:
             self.driver_dbs = [detection.LocalKernelModulesDriverDB(),
                 detection.OpenPrintingDriverDB()]
-            self._detect_handlers()
+            self.handlers = {}
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
         in_signature='s', out_signature='as', sender_keyword='sender',
@@ -243,8 +269,8 @@ class Backend(dbus.service.Object):
             v = getattr(h, f)()
             if v:
                 info[f] = v
-        for f in ['package', 'repository', 'driver_vendor', 'version',
-            'license']:
+        for f in ['package', 'repository', 'repository_sign_fp',
+                'driver_vendor', 'version', 'license']:
             v = getattr(h, f)
             if v:
                 info[f] = v
@@ -331,6 +357,10 @@ class Backend(dbus.service.Object):
                     self.remove_progress(-1, -1)
         if self._f_exception:
             raise self._f_exception[0], self._f_exception[1], self._f_exception[2]
+
+        # notify about reboot
+        if not self._f_result:
+            OSLib.inst.notify_reboot_required()
         return self._f_result
 
     @dbus.service.method(DBUS_INTERFACE_NAME,
@@ -454,12 +484,12 @@ class Backend(dbus.service.Object):
         '''
         try:
             cls = getattr(detection, db_class)
-        except AttributeError, e:
+        except AttributeError as e:
             raise InvalidDriverDBException(str(e))
 
         try:
             inst = cls(*db_args)
-        except Exception, e:
+        except Exception as e:
             raise InvalidDriverDBException(str(e))
 
         logging.debug('add_driverdb: Adding %s', str(inst))
@@ -590,7 +620,7 @@ class Backend(dbus.service.Object):
     # Internal API for calling from Handlers (not exported through D-BUS)
     #
 
-    def install_package(self, package):
+    def install_package(self, package, repository=None, fingerprint=None):
         '''Install software package.'''
 
         if OSLib.inst.package_installed(package):
@@ -599,7 +629,8 @@ class Backend(dbus.service.Object):
         # only pass D-BUS signal callback if we are called as a D-BUS backend
         self._package_operation_in_progress = True
         OSLib.inst.install_package(package, 
-            hasattr(self, '_locations') and self.install_progress or None)
+            hasattr(self, '_locations') and self.install_progress or None,
+            repository, fingerprint)
         if OSLib.inst.package_installed(package):
             self._update_installed_packages([package], [])
         self._package_operation_in_progress = False
@@ -782,7 +813,7 @@ class Backend(dbus.service.Object):
                     ('unix-process', {'pid': dbus.UInt32(pid, variant_level=1),
                         'start-time': dbus.UInt64(0, variant_level=1)}), 
                     privilege, {'': ''}, dbus.UInt32(1), '', timeout=600)
-        except dbus.DBusException, e:
+        except dbus.DBusException as e:
             if e._dbus_error_name == 'org.freedesktop.DBus.Error.ServiceUnknown':
                 # polkitd timed out, connect again
                 self.polkit = None
