@@ -24,9 +24,11 @@ unknown hardware.
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import os, os.path, subprocess, sys, logging, re
+import os, os.path, subprocess, sys, logging, re, urllib2
 import cPickle, xmlrpclib, locale
 from glob import glob
+from cStringIO import StringIO
+import pycurl
 
 from oslib import OSLib
 import handlers, xorg_driver
@@ -110,6 +112,7 @@ class DriverID:
     - version: string, arbitrary differentiation [optional]
     - jockey_handler: class name (inferred from driver_type for standard handlers) [optional]
     - repository: URL [optional]
+    - fingerprint: GPG fingerprint of repository [for binary packages, if repository is present]
     - package: string [optional]
     - free: boolean (licensed as free software) [required]
     - license: string (license text) [required if free is False]
@@ -154,7 +157,7 @@ class DriverDB:
                 'driverdb-%s.cache' % self._cache_id())
             try:
                 self.cache = cPickle.load(open(self.cache_path, 'rb'))
-            except (cPickle.PickleError, IOError), e:
+            except (cPickle.PickleError, IOError) as e:
                 logging.warning('Could not open DriverDB cache %s: %s',
                     self.cache_path, str(e))
                 self.cache = None
@@ -186,7 +189,7 @@ class DriverDB:
                 f = open(self.cache_path, 'wb')
                 cPickle.dump(self.cache, f)
                 f.close()
-            except (cPickle.PickleError, IOError), e:
+            except (cPickle.PickleError, IOError) as e:
                 logging.warning('Could not create DriverDB cache %s: %s',
                     self.cache_path, str(e))
 
@@ -213,7 +216,7 @@ class DriverDB:
         This should not be called directly. Users call query() which wraps this
         function into caching.
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def _do_update(self, hwids):
         '''Query remote server for driver updates for a set of HardwareIDs.
@@ -263,6 +266,25 @@ class LocalKernelModulesDriverDB(DriverDB):
         # patterns for which we can optimize lookup
         self.vendor_pattern_re = re.compile('(pci|usb):v([0-9A-F]{4,8})(?:d|p)')
 
+        # first, read stuff from package headers
+        # structure: package_name -> module_name -> [list of modaliases]
+        for pkg, ma_map in OSLib.inst.package_header_modaliases().iteritems():
+            for module, aliases in ma_map.iteritems():
+                for alias in aliases:
+                    vp = self.vendor_pattern_re.match(alias)
+                    if vp:
+                        self.alias_cache.setdefault(vp.group(1), {}).setdefault(
+                            vp.group(2), {}).setdefault(alias, []).append((module, pkg))
+                    else:
+                        colon = alias.find(':')
+                        if colon > 0:
+                            bus = alias[:colon]
+                        else:
+                            bus = None
+                        self.alias_cache.setdefault(bus, {}).setdefault(
+                            None, {}).setdefault(alias, []).append((module, pkg))
+
+        # local modalias lists override package header information
         for alias_location in OSLib.inst.modaliases:
             if not os.path.exists(alias_location):
                 continue
@@ -341,6 +363,9 @@ class LocalKernelModulesDriverDB(DriverDB):
                     for (m, p) in mods:
                         did = DriverID(driver_type='kernel_module', kernel_module=m)
                         if p:
+                            if m == 'hwe':
+                                did.properties.pop('kernel_module')
+                                did.properties['driver_type'] = 'hwe'
                             did.properties['package'] = p
                         result.append(did)
         else:
@@ -352,6 +377,9 @@ class LocalKernelModulesDriverDB(DriverDB):
                 for (m, p) in mods:
                     did = DriverID(driver_type='kernel_module', kernel_module=m)
                     if p:
+                        if m == 'hwe':
+                            did.properties.pop('kernel_module')
+                            did.properties['driver_type'] = 'hwe'                        
                         did.properties['package'] = p
                     result.append(did)
 
@@ -484,7 +512,7 @@ class OpenPrintingDriverDB(DriverDB):
             elif pkgname.endswith('.rpm'):
                 return '-'.join(pkgname.split('-')[0:-2])
             else:
-                raise ValueError, 'Unknown package type: ' + pkgname
+                raise ValueError('Unknown package type: ' + pkgname)
 
         def _ld_callback(status, drv_list, data):
             if status != 0:
@@ -492,8 +520,11 @@ class OpenPrintingDriverDB(DriverDB):
                 return
 
             for driver, info in data.iteritems():
+                info_shortlicense = info.copy()
+                if 'licensetext' in info_shortlicense:
+                    info_shortlicense['licensetext'] = info_shortlicense['licensetext'][:20] + '..'
                 logging.debug('OpenPrintingDriverDB: driver %s info: %s',
-                    driver, str(info))
+                    driver, str(info_shortlicense))
                 pkgs = info.get('packages', {})
                 arches = pkgs.keys()
                 if len(arches) == 0:
@@ -502,6 +533,7 @@ class OpenPrintingDriverDB(DriverDB):
                 if len(arches) > 1:
                     logging.error('Returned more than one matching architecture, please report this as a bug: %s', str(arches))
                     continue
+
                 pkgs = pkgs[arches[0]]
 
                 if len(pkgs) != 1:
@@ -509,6 +541,18 @@ class OpenPrintingDriverDB(DriverDB):
                     return
                 pkg = pkgs.keys()[0]
 
+                # require signature for binary packages; architecture
+                # independent packages are usually PPDs, which we trust enough
+                fingerprint = None
+                if arches[0] not in ['all', 'noarch']: 
+                    if 'fingerprint' not in pkgs[pkg]:
+                        logging.debug('Ignoring driver as it does not have a GPG fingerprint URL')
+                        continue
+                    fingerprint = download_gpg_fingerprint(pkgs[pkg]['fingerprint'])
+                    if not fingerprint:
+                        logging.debug('Ignoring driver as it does not have a valid GPG fingerprint')
+                        continue
+
                 repo = pkgs[pkg].get('repositories', {}).get(pkgsystem)
                 if not repo:
                     logging.error('Local package system %s not found in %s',
@@ -549,6 +593,11 @@ class OpenPrintingDriverDB(DriverDB):
                 if driver != did.properties['package']:
                     did.properties['version'] = driver.replace('driver/', '')
 
+                if fingerprint:
+                    did.properties['fingerprint'] = fingerprint
+
+                logging.debug('Created DriverID: ' + str(did.properties))
+
                 if 'licensetext' in info:
                     did.properties['license'] = info['licensetext']
 
@@ -567,12 +616,13 @@ class OpenPrintingDriverDB(DriverDB):
             logging.debug('   ... querying for %s', hwid.id)
             t = op.listDrivers(hwid.id, _ld_callback,
                 self.cache.setdefault(hwid, []), 
-                extra_options={'packagesystem': opo_pkgsystem, 'architectures': 'noarch'})
+                extra_options={'packagesystem': opo_pkgsystem})
             threads.append(t)
 
         # wait until all threads have finished
         for t in threads:
             t.join()
+        logging.debug('openprinting.org database query finished')
 
 #--------------------------------------------------------------------#
 # internal helper functions
@@ -615,40 +665,6 @@ def _get_modaliases():
 
 _get_modaliases.cache = None
 
-def _get_printers():
-    '''Return a set of HardwareIDs for connected printers.'''
-
-    if _get_printers.cache is not None:
-        return _get_printers.cache
-
-    _get_printers.cache = set()
-
-    try:
-        import cups
-        import cupshelpers
-    except ImportError:
-        logging.warning('cups and/or cupshelpers Python modules are not present; printer detection is not available')
-        return _get_printers.cache
-
-    try:
-        for dev in cupshelpers.getDevices(cups.Connection()).itervalues():
-            # openprinting.org database only uses MFG, MDL, DES, and CMD, so don't
-            # send the rest (it might contain personal data such as serial numbers)
-            if dev.id_dict.get('MFG') and dev.id_dict.get('MDL'):
-                id = 'MFG:%s;MDL:%s;' % (dev.id_dict['MFG'], dev.id_dict['MDL'])
-                if dev.id_dict.get('DES'):
-                    id += 'DES:' + dev.id_dict['DES'] + ';'
-                if dev.id_dict.get('CMD'):
-                    id += 'CMD:' + ','.join(dev.id_dict['CMD']) + ';'
-                _get_printers.cache.add(HardwareID('printer_deviceid', id))
-    except (RuntimeError, cups.IPPError):
-        logging.warning('cannot connect to cups; printer detection is not available')
-        return set()
-           
-    return _get_printers.cache
-
-_get_printers.cache = None
-
 def _handler_license_filter(handler, mode):
     '''Filter handlers by license.
     
@@ -676,6 +692,8 @@ def _driverid_to_handler(did, backend, mode):
             did['jockey_handler'] = 'KernelModuleHandler'
         elif did['driver_type'] == 'printer_driver':
             did['jockey_handler'] = 'PrinterDriverHandler'
+        elif did['driver_type'] == 'hwe':
+            did['jockey_handler'] = 'HWEHandler'
         else:
             logging.warning('Cannot map driver type %s to a default handler' %
                 did['driver_type'])
@@ -687,8 +705,8 @@ def _driverid_to_handler(did, backend, mode):
     # special treatment of some standard attributes: those must be applied
     # to the particular handler instance
     for a in ('driver_type', 'jockey_handler', 'version', 'description',
-        'long_description', 'repository', 'package', 'driver_vendor', 'free',
-        'license', 'recommended'):
+        'long_description', 'repository', 'fingerprint', 'package',
+        'driver_vendor', 'free', 'license', 'recommended'):
         try:
             del kwargs[a]
         except KeyError:
@@ -720,13 +738,21 @@ def _driverid_to_handler(did, backend, mode):
                 # only create default handlers for modules which actually
                 # exist or which we can install
                 if not get_modinfo(did['kernel_module']):
-                    if 'free' not in did.properties or \
-                       'description' not in did.properties or \
-                       'package' not in did.properties:
-                       logging.warning('DriverID for module %s does not refer '
-                           'to a locally available module and does not specify '
-                           'free/description/package; ignoring', did['kernel_module'])
-                       return None
+                    if 'package' not in did.properties:
+                        logging.warning('Module %s does not specify package', did['kernel_module'])
+                        return None
+                    try:
+                        if 'free' not in did.properties:
+                            did.properties['free'] = OSLib.inst.is_package_free(did.properties['package'])
+                        if 'description' not in did.properties:
+                            short = {}
+                            long = {}
+                            (short['C'], long['C'])  = OSLib.inst.package_description(did.properties['package'])
+                            did.properties['description'] = short
+                            did.properties['long_description'] = long
+                    except ValueError as e:
+                        logging.warning('Cannot determine properties of package %s: %s' % (did.properties['package'], str(e)))
+                        return None
 
                 hclass = getattr(handlers, did['jockey_handler'])
 
@@ -752,6 +778,14 @@ def _driverid_to_handler(did, backend, mode):
         args.append('dummy') # name is overridden later
         kwargs['description'] = ''
 
+    # instantiation of HWE handlers
+    if did['jockey_handler'] == 'HWEHandler':
+        hclass = getattr(handlers, did['jockey_handler'])
+        if not 'package' in did.properties:
+            logging.warning('Hardware Enablement handler requires specifying a package')
+        package = did.properties['package']
+        kwargs['package'] = package
+
     # TODO: HandlerGroup standard handlers
 
     # default: look up handler in the handler pool
@@ -767,7 +801,7 @@ def _driverid_to_handler(did, backend, mode):
     # create instance and set specific properties from DriverID
     try:
         h = hclass(backend, *args, **kwargs)
-    except Exception, e:
+    except Exception as e:
         logging.warning('could not instantiate handler class %s with args %s and kwargs %s: %s', 
             str(hclass), str(args), str(kwargs), str(e))
         return None
@@ -784,6 +818,8 @@ def _driverid_to_handler(did, backend, mode):
         h.package = did.properties['package']
     if 'repository' in did.properties:
         h.repository = did.properties['repository']
+    if 'fingerprint' in did.properties:
+        h.repository_sign_fp = did.properties['fingerprint']
     if 'free' in did.properties:
         h._free = did.properties['free']
     if 'license' in did.properties:
@@ -848,8 +884,6 @@ def get_hardware():
 
     # modaliases
     result = _get_modaliases()
-    # printer device IDs
-    result.update(_get_printers())
 
     # other hardware detection goes here
 
@@ -925,6 +959,8 @@ def get_handlers(backend, driverdb=None, handler_dir=None, mode=MODE_ANY,
                     logging.debug('%s does not match license mode %i', str(obj), mode)
                     continue
 
+                backend.handler_pool[name] = inst
+
                 avail = (not available_only) or inst.available()
                 if avail:
                     if hardware_only:
@@ -932,10 +968,8 @@ def get_handlers(backend, driverdb=None, handler_dir=None, mode=MODE_ANY,
                     else:
                         logging.debug('%s is available', desc)
                         available_handlers.add(inst)
-                    backend.handler_pool[name] = inst
                 elif avail == None:
                     logging.debug('%s availability undetermined, adding to pool', desc)
-                    backend.handler_pool[name] = inst
                 else:
                     logging.debug('%s not available', desc)
 
@@ -973,7 +1007,6 @@ def get_db_handlers(backend, db, hardware, mode=MODE_ANY):
         if len(dids) == 1:
             try:
                 del dids[0].properties['recommended']
-                logging.debug('DriverID %s for %s is the only alternative, removed recommendation', str(dids[0].properties), hwid)
             except KeyError:
                 pass
 
@@ -992,4 +1025,41 @@ def get_db_handlers(backend, db, hardware, mode=MODE_ANY):
 
     return available_handlers
 
+def download_gpg_fingerprint(url):
+    '''Get GPG fingerprint from URL.
 
+    Check that the URL is HTTPS with a valid and trusted server
+    certificate, read it, extract the GPG fingerprint from it, and return
+    it. Return None if the URL is invalid, not trusted, or the fingerprint
+    can't be found.
+    '''
+    if not url.startswith('https://'):
+        logging.debug('Not a https fingerprint URL: %s, ignoring driver' % url)
+        return None
+
+    cert = OSLib.inst.ssl_cert_file()
+    if not cert:
+        logging.debug('No system SSL certificates available for trust checking')
+        return None
+
+    c = pycurl.Curl()
+    c.setopt(pycurl.URL, url)
+    content = StringIO()
+    c.setopt(pycurl.WRITEFUNCTION, content.write)
+    c.setopt(pycurl.FOLLOWLOCATION, 1)
+    c.setopt(pycurl.MAXREDIRS, 5)
+    c.setopt(pycurl.CAINFO, cert)
+
+    try:
+        c.perform()
+    except pycurl.error as e:
+        logging.warning('Cannot retrieve %s: %s' % (url, str(e)))
+        return None
+
+    fingerprint_re = re.compile(' ((?:(?:[0-9A-F]{4})(?:\s+|$)){10})$', re.M)
+    
+    m = fingerprint_re.search(content.getvalue())
+    if m:
+        return m.group(1).strip()
+
+    return None
