@@ -76,6 +76,7 @@ class Handler:
         self.driver_vendor = None
         self.version = None
         self.repository = None
+        self.repository_sign_fp = None
         self._recommended = False
         self._auto_install = False
 
@@ -164,7 +165,7 @@ class Handler:
         if self._free is not None:
             return self._free
         else:
-            raise NotImplementedError, 'subclasses need to implement this'
+            raise NotImplementedError('subclasses need to implement this')
 
     def auto_install(self):
         '''Return if the handler should be automatically installed.
@@ -197,7 +198,7 @@ class Handler:
     def used(self):
         '''Return if the handler is currently in use.'''
 
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def recommended(self):
         '''Return if the version of a certain driver is recommended over others
@@ -231,7 +232,7 @@ class Handler:
             else:
                 return None # undecidable until the repo is added
         else:
-            raise NotImplementedError, 'subclasses need to implement this'
+            raise NotImplementedError('subclasses need to implement this')
 
     def enable(self):
         '''Allow the OS to use it if the hardware is available.
@@ -240,11 +241,9 @@ class Handler:
         immediately successful, or False if the system needs to be rebooted for
         the changes to become effective.
         '''
-        if self.repository:
-            OSLib.inst.add_repository(self.repository)
-
         if self.package:
-            self.backend.install_package(self.package)
+            self.backend.install_package(self.package, self.repository,
+                    self.repository_sign_fp)
             if not OSLib.inst.package_installed(self.package):
                 # do not touch _changed if package failed to install
                 return True
@@ -259,11 +258,7 @@ class Handler:
         If possible, the handler should be unloaded, too. Return True if
         immediately successful, or False if the system needs to be rebooted for
         the changes to become effective.
-
         '''
-        if self.repository:
-            OSLib.inst.remove_repository(self.repository)
-
         if self.package:
             self.backend.remove_package(self.package)
             if OSLib.inst.package_installed(self.package):
@@ -624,7 +619,7 @@ class FirmwareHandler(KernelModuleHandler):
         This downloads the url and puts it into self.firmware_file. Subclasses
         need to provide an actual implementation what to do with the file.
         '''
-        raise NotImplementedError, 'FirmwareHandler is currently not implemented'
+        raise NotImplementedError('FirmwareHandler is currently not implemented')
         #self.firmware_file = self.ui.download_url(self.url)[0]
         #if not self.firmware_file:
         #    return
@@ -639,7 +634,7 @@ class FirmwareHandler(KernelModuleHandler):
         Implementation in subclasses need to remove the firmware files and call
         KernelModuleHandler.disable().
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
 #--------------------------------------------------------------------#
 
@@ -667,3 +662,18 @@ class PrinterDriverHandler(Handler):
 
         # TODO: query cups for actually using the driver
         return self.enabled()
+
+class HWEHandler(Handler):
+    '''Handler for general hardware enablement.
+    
+    This is for general quirks which aren't kernel modules. This uses aliases
+    in a fashion similar to KernelModuleHandler, but ties directly to package
+    names instead.
+    '''
+    def __init__(self, backend, package):
+        self.package = package
+        Handler.__init__(self, backend, '')
+
+    def used(self):
+        return self.enabled()
+
