@@ -17,7 +17,7 @@
 
 '''Encapsulate operations which are Linux distribution specific.'''
 
-import fcntl, os, subprocess, sys, logging, re, tempfile, time
+import fcntl, os, subprocess, sys, logging, re, tempfile, time, shutil
 from glob import glob
 
 class OSLib:
@@ -80,6 +80,21 @@ class OSLib:
 
         self._load_module_blacklist()
 
+        # Possible paths of a file with a set of SSL certificates which are
+        # considered trustworthy. The first one that exists will be used.
+        # This is used for downloading GPG key fingerprints for
+        # openprinting.org driver packages.
+        self.ssl_cert_file_paths = [
+                # Debian/Ubuntu use the ca-certificates package:
+                '/etc/ssl/certs/ca-certificates.crt'
+                ]
+
+        # default GPG key server
+        # this is the generally recommended DNS round-robin, but usually very
+        # slow:
+        #self.gpg_key_server = 'keys.gnupg.net'
+        self.gpg_key_server = 'keyserver.ubuntu.com'
+
     # 
     # The following package related functions use PackageKit; if that does not
     # work for your distribution, they must be reimplemented
@@ -100,7 +115,7 @@ class OSLib:
             return m.group(1).lower().startswith('gpl') or \
                 m.group(1).lower() in ('free', 'bsd', 'mpl')
         else:
-            raise ValueError, 'package %s does not exist' % package
+            raise ValueError('package %s does not exist' % package)
 
     def package_installed(self, package):
         '''Return if the given package is installed.'''
@@ -126,7 +141,7 @@ class OSLib:
             # TODO: short description (not accessible with pkcon)
             return (package, m.group(1))
         else:
-            raise ValueError, 'package %s does not exist' % package
+            raise ValueError('package %s does not exist' % package)
 
     def package_files(self, package):
         '''Return a list of files shipped by a package.
@@ -142,9 +157,10 @@ class OSLib:
         if pkcon.returncode == 0 and '\n  ' in out:
             return [l.strip() for l in out.splitlines() if l.startswith('  ')]
         else:
-            raise ValueError, 'package %s is not installed' % package
+            raise ValueError('package %s is not installed' % package)
 
-    def install_package(self, package, progress_cb):
+    def install_package(self, package, progress_cb, repository=None,
+            fingerprint=None):
         '''Install the given package.
 
         As this is called in the backend, this must happen noninteractively.
@@ -157,8 +173,25 @@ class OSLib:
         If this succeeds, subsequent package_installed(package) calls must
         return True.
 
-        Any installation failure should be raised as a SystemError.
+        If a repository URL is given, that repository is added to the system
+        first. The format for repository is distribution specific. This function
+        should also download/update the package index for this repository when
+        adding it.
+        .
+        fingerprint, if not None, is a GPG-style fingerprint of that
+        repository; if present, this method should also retrieve that GPG key
+        from the keyservers, install it into the packaging system, and ensure
+        that the repository is signed with that key.
+
+        An unknown package should raise a ValueError. Any installation failure
+        should be raised as a SystemError.
         '''
+        if repository or fingerprint:
+            raise NotImplementedError('PackageKit default implementation does not currently support repositories or fingerprints')
+
+        # this will check if the package exists
+        self.package_description(package)
+
         pkcon = subprocess.Popen(['pkcon', 'install', '-v', package],
             stdout=subprocess.PIPE, stderr=subprocess.PIPE)
 
@@ -188,7 +221,7 @@ class OSLib:
 
         err += pkcon.stderr.read()
         if pkcon.wait() != 0 or not self.package_installed(package):
-            raise SystemError, 'package %s failed to install: %s' % (package, err) 
+            raise SystemError('package %s failed to install: %s' % (package, err)) 
             
     def remove_package(self, package, progress_cb):
         '''Uninstall the given package.
@@ -231,7 +264,7 @@ class OSLib:
         err += pkcon.stderr.read()
         pkcon.wait()
         if self.package_installed(package):
-            raise SystemError, 'package %s failed to remove: %s' % (package, err) 
+            raise SystemError('package %s failed to remove: %s' % (package, err)) 
 
     def has_repositories(self):
         '''Check if package repositories are available.
@@ -277,7 +310,83 @@ class OSLib:
         elif os.path.exists('/etc/yum.conf'):
             return 'yum'
 
-        raise NotImplementedError, 'local packaging system is unknown'
+        raise NotImplementedError('local packaging system is unknown')
+
+    def import_gpg_key(self, keyring, fingerprint):
+        '''Download and install a GPG key identified by fingerprint.
+
+        This verifies that the fingerprint matches, and if so, installs it into
+        the given keyring file (will be created if it does not exist).
+
+        Raise a SystemError if anything goes wrong.
+        '''
+        if fingerprint in self._gpg_keyring_fingerprints(keyring):
+            return
+
+        # gpg likes to write trustdb and temporary files, so create a temporary
+        # home directory
+        keyid = fingerprint.replace(' ', '')[-8:]
+        gpghome = tempfile.mkdtemp()
+        default_keyring = os.path.join(gpghome, 'pubring.gpg')
+        try:
+            # we first import into a temporary keyring, as we need to verify
+            # the fingerprint
+            gpg = subprocess.Popen(['gpg', '--homedir', gpghome,
+                '--no-default-keyring', '--primary-keyring', default_keyring,
+                '--keyserver', self.gpg_key_server, '--recv-key', keyid], 
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE, 
+                env={'PATH': os.environ.get('PATH', '')})
+            (out, err) = gpg.communicate()
+
+            if fingerprint not in self._gpg_keyring_fingerprints(default_keyring):
+                raise SystemError('gpg failed to import key: ' + err)
+
+            # now move over to the actual keyring; for multiple matches of the
+            # same key ID it would be great to be able to select the one that
+            # we want, but unfortunately the GPG command line doesn't really
+            # allow that; fortunately key ID conflicts are very rare.
+            gpg = subprocess.Popen(['gpg', '--homedir', gpghome,
+                '--no-default-keyring', '--primary-keyring', keyring,
+                '--import', default_keyring], 
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+                env={'PATH': os.environ.get('PATH', '')})
+            (out, err) = gpg.communicate()
+
+            if fingerprint not in self._gpg_keyring_fingerprints(default_keyring):
+                raise SystemError('gpg failed to import key: ' + err)
+
+            logging.debug('import_gpg_key(): Successfully imported key' + keyid)
+        except OSError as e:
+            raise SystemError('failed to call gpg: ' + str(e))
+        finally:
+            shutil.rmtree(gpghome)
+
+    def _gpg_keyring_fingerprints(self, keyring):
+        '''Return list of fingerprints in given keyring'''
+
+        # gpg likes to write trustdb and temporary files, so create a temporary
+        # home directory
+        gpghome = tempfile.mkdtemp()
+        try:
+            result = []
+            gpg = subprocess.Popen(['gpg', '--homedir', gpghome,
+                '--no-default-keyring', '--primary-keyring', keyring,
+                '--fingerprint'], stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE, env={})
+            (out, err) = gpg.communicate()
+            if gpg.returncode != 0:
+                logging.error('failed to call gpg for reading fingerprints: ' + err)
+                return []
+
+            for l in out.splitlines():
+                if 'fingerprint =' in l:
+                    result.append(l.split('=')[1].strip())
+            return result
+        except OSError as e:
+            logging.error('failed to call gpg: ' + str(e))
+            return []
+        finally:
+            shutil.rmtree(gpghome)
 
     # 
     # The following functions MUST be implemented by distributors
@@ -285,28 +394,10 @@ class OSLib:
     # RepoSetData(), so those need to remain package system specific
     #
 
-    def add_repository(self, repository):
-        '''Add a repository.
-
-        The format for repository is distribution specific. This function
-        should also download/update the package index for this repository.
-
-        This should throw a ValueError if the repository is invalid or
-        inaccessible.
-        '''
-        raise NotImplementedError, 'subclasses need to implement this'
-
-    def remove_repository(self, repository):
-        '''Remove a repository.
-
-        The format for repository is distribution specific.
-        '''
-        raise NotImplementedError, 'subclasses need to implement this'
-
     def repository_enabled(self, repository):
         '''Check if given repository is enabled.'''
 
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_help_available(self, ui):
         '''Return if help is available.
@@ -339,7 +430,7 @@ class OSLib:
         if not os.path.isdir(self.backup_dir):
             try:
                 os.makedirs(self.backup_dir)
-            except OSError, e:
+            except OSError as e:
                 logging.error('Could not create %s: %s, using temporary '
                     'directory; all your caches will be lost!',
                     self.backup_dir, str(e))
@@ -426,7 +517,7 @@ class OSLib:
                 os.unlink(self.module_blacklist_file)
                 return
 
-        os.umask(022)
+        os.umask(0o22)
         # create directory if it does not exist
         d = os.path.dirname(self.module_blacklist_file)
         if not os.path.exists(d):
@@ -473,3 +564,84 @@ class OSLib:
             product = ''
 
         return (vendor, product)
+
+    def notify_reboot_required(self):
+        '''Notify the system that a reboot is required.
+
+        This can be used as an extra indication when installing a driver which
+        needs a reboot to get active.
+
+        The default implementation does nothing.
+        '''
+        pass
+
+    def package_header_modaliases(self):
+        '''Get modalias map from package headers.
+
+        Driver packages may declare the modaliases that they support in a
+        package header field, so that they do not need to have a separate
+        modalias file list already installed. The map must have the following
+        structure: package_name -> module_name -> [list of modaliases]
+
+        If this is not supported, simply return an empty dictionary here.
+        '''
+        return {}
+
+    def ssl_cert_file(self):
+        '''Get file with trusted SSL certificates.
+        
+        This is used for downloading GPG key fingerprints for
+        openprinting.org driver packages.
+
+        Return None if no certificates file is available.
+        '''
+        for f in self.ssl_cert_file_paths:
+            if os.path.exists(f):
+                return f
+
+        return None
+
+    @classmethod
+    def has_defaultroute(klass):
+        '''Return if there is a default route.
+
+        This is a reasonable indicator that online tests can be run.
+        '''
+        if klass._has_defaultroute_cache is None:
+            klass._has_defaultroute_cache = False
+            route = subprocess.Popen(['/sbin/route', '-n'],
+                stdout=subprocess.PIPE)
+            for l in route.stdout:
+                if l.startswith('0.0.0.0 '):
+                    klass._has_defaultroute_cache = True
+            route.wait()
+
+        return klass._has_defaultroute_cache
+
+    _has_defaultroute_cache = None
+
+    def current_xorg_video_abi(self):
+        '''Return current X.org video ABI.
+
+        For an X.org video driver to actually work it must be built against the
+        currently used X.org driver ABI, otherwise it will cause crashes. This
+        method returns the currently expected video driver ABI from the X
+        server. If it is not None, it must match video_driver_abi() of a driver
+        package for this driver to be offered for installation.
+        
+        If this returns None, ABI checking is disabled.
+        '''
+        return None
+
+    def video_driver_abi(self, package):
+        '''Return video ABI for an X.org driver package.
+
+        For an X.org video driver to actually work it must be built against the
+        currently used X.org driver ABI, otherwise it will cause crashes. This
+        method returns the video ABI for a driver package. If it is not None,
+        it must match current_xorg_video_abi() for this driver to be offered
+        for installation.
+        
+        If this returns None, ABI checking is disabled.
+        '''
+        return None
