@@ -28,8 +28,10 @@ import dbus.service
 import dbus.mainloop.glib
 
 try:
-    import appindicator
+    from gi.repository import AppIndicator
 except:
+    # UIs will need to implement their own status icon if app indicator isn't
+    # available
     pass
 
 from oslib import OSLib
@@ -45,20 +47,6 @@ def bool(str):
     assert str == 'False'
     return False
 
-# Avoid having to do .encode('UTF-8') everywhere. This is a pain; I wish
-# Python supported something like "sys.stdout.encoding = 'UTF-8'".
-def __fix_stdouterr():
-    import codecs
-    import locale
-    def null_decode(input, errors='strict'):
-        return input, len(input)
-    sys.stdout = codecs.EncodedFile(sys.stdout, locale.getpreferredencoding())
-    sys.stdout.decode = null_decode
-    sys.stderr = codecs.EncodedFile(sys.stderr, locale.getpreferredencoding())
-    sys.stderr.decode = null_decode
-
-__fix_stdouterr()
-
 class AbstractUI(dbus.service.Object):
     '''Abstract user interface.
 
@@ -75,6 +63,9 @@ class AbstractUI(dbus.service.Object):
 
         (self.argv_options, self.argv_args) = self.parse_argv()
 
+        if not OSLib.inst:
+            OSLib.inst = OSLib(client_only=not self.argv_options.no_dbus)
+
         if self.argv_options.check:
             time.sleep(self.argv_options.check)
 
@@ -83,6 +74,7 @@ class AbstractUI(dbus.service.Object):
         self._dbus_iface = None
         self.dbus_server_main_loop = None
         self.have_ui = False
+        self.search_only = False
         self.current_search = (None, None) # query, result
 
         # make Control-C work properly
@@ -92,11 +84,18 @@ class AbstractUI(dbus.service.Object):
         '''Return D-BUS backend client interface.
 
         This gets initialized lazily.
+
+        Set self.search_only to True to suppress a full system hardware
+        detection, especially if you use only search_driver() to
+        find a remote driver for a selected, already detected device.
         '''
         if self._dbus_iface is None:
             try:
-                self._dbus_iface = Backend.create_dbus_client()
-            except Exception, e:
+                if self.argv_options.no_dbus:
+                    self._dbus_iface = Backend()
+                else:
+                    self._dbus_iface = Backend.create_dbus_client()
+            except Exception as e:
                 if hasattr(e, '_dbus_error_name') and e._dbus_error_name == \
                     'org.freedesktop.DBus.Error.FileNotFound':
                     if self.have_ui:
@@ -110,19 +109,21 @@ class AbstractUI(dbus.service.Object):
             self._check_repositories()
             self._call_progress_dialog(
                 self._('Searching for available drivers...'),
-                self._dbus_iface.detect, timeout=600)
+                self.search_only and self._dbus_iface.db_init or self._dbus_iface.detect, 
+                timeout=600)
         else:
             # handle backend timeouts
             try:
                 self._dbus_iface.handler_info(' ')
-            except Exception, e:
+            except Exception as e:
                 if hasattr(e, '_dbus_error_name') and e._dbus_error_name == \
                     'org.freedesktop.DBus.Error.ServiceUnknown':
                     self._dbus_iface = Backend.create_dbus_client()
                     self._check_repositories()
                     self._call_progress_dialog(
                         self._('Searching for available drivers...'),
-                        self._dbus_iface.detect, timeout=600)
+                        self.search_only and self._dbus_iface.db_init or self._dbus_iface.detect, 
+                        timeout=600)
 
         return self._dbus_iface
 
@@ -371,6 +372,8 @@ class AbstractUI(dbus.service.Object):
         parser.add_option ('--dbus-server', action='store_true',
                 dest='dbus_server', default=False,
                 help=self._('Run as session D-BUS server.'))
+        parser.add_option ('--no-dbus', action='store_true', default=False,
+                help=self._('Do not use D-BUS for communicating with the backend. Needs root privileges.'))
         #parser.add_option ('--debug', action='store_true',
         #        dest='debug', default=False,
         #        help=self._('Enable debugging messages.'))
@@ -449,7 +452,7 @@ class AbstractUI(dbus.service.Object):
         for h_id in self.backend().available(self.argv_options.mode):
             i = self.backend().handler_info(h_id)
             print '%s - %s (%s, %s, %s)%s' % (
-                h_id, i['name'],
+                h_id, self._(i['name']),
                 bool(i['free']) and self.string_free or self.string_restricted,
                 bool(i['enabled']) and self.string_enabled or self.string_disabled,
                 bool(i['used']) and self.string_in_use or self.string_not_in_use,
@@ -471,7 +474,7 @@ class AbstractUI(dbus.service.Object):
         # time out on this.
         try:
             convert_dbus_exceptions(self.backend)
-        except Exception, e:
+        except Exception as e:
             print >> sys.stderr, 'Cannot connect to backend, is it busy?\n', e
             return False
 
@@ -552,7 +555,7 @@ class AbstractUI(dbus.service.Object):
 
     def _install_progress_handler(self, phase, cur, total):
         if not self._install_progress_shown:
-            self.ui_progress_start('', 
+            self.ui_progress_start(self._('Additional Drivers'), 
                 self._('Downloading and installing driver...'), total)
             self._install_progress_shown = True
         self.ui_progress_update(cur, total)
@@ -560,7 +563,7 @@ class AbstractUI(dbus.service.Object):
 
     def _remove_progress_handler(self, cur, total):
         if not self._install_progress_shown:
-            self.ui_progress_start('', 
+            self.ui_progress_start(self._('Additional Drivers'), 
                 self._('Removing driver...'), total)
             self._install_progress_shown = True
         self.ui_progress_update(cur, total)
@@ -568,7 +571,7 @@ class AbstractUI(dbus.service.Object):
 
     def _repository_progress_handler(self, cur, total):
         if not self._repository_progress_shown:
-            self.ui_progress_start('', 
+            self.ui_progress_start(self._('Additional Drivers'), 
                 self._('Downloading and updating package indexes...'), total)
             self._repository_progress_shown = True
         self.ui_progress_update(cur, total)
@@ -602,7 +605,7 @@ class AbstractUI(dbus.service.Object):
         elif action == 'toggle':
             enable = not bool(i['enabled'])
         else:
-            raise ValueError, 'invalid action %s; allowed are enable, disable, toggle'
+            raise ValueError('invalid action %s; allowed are enable, disable, toggle')
 
         # check if we can change at all
         if 'can_change' in i:
@@ -656,7 +659,7 @@ class AbstractUI(dbus.service.Object):
                 self._('Sorry, the Jockey backend crashed. Please file a bug at:'),
                 self._('Trying to recover by restarting backend.')))
             return False
-        except SystemError, e:
+        except SystemError as e:
             self.error_message('', str(e).strip().splitlines()[-1])
             return False
 
@@ -702,7 +705,7 @@ class AbstractUI(dbus.service.Object):
         current_size = 0
         try:
             f = urllib2.urlopen(url)
-        except Exception, e:
+        except Exception as e:
             self.error_message(self._('Download error'), str(e))
             return (None, None)
         headers = f.info()
@@ -753,6 +756,12 @@ class AbstractUI(dbus.service.Object):
     def _call_progress_dialog(self, message, fn, *args, **kwargs):
         '''Call fn(*args, **kwargs) while showing a progress dialog.'''
 
+        if self.argv_options.no_dbus:
+            try:
+                del kwargs['timeout']
+            except KeyError:
+                pass
+
         if not self.have_ui:
             return fn(*args, **kwargs)
 
@@ -766,7 +775,7 @@ class AbstractUI(dbus.service.Object):
                 break
             if not progress_shown:
                 progress_shown = True
-                self.ui_progress_start('', message, -1)
+                self.ui_progress_start(self._('Additional Drivers'), message, -1)
             if self.ui_progress_update(-1, -1):
                 sys.exit(1) # cancel
             self.ui_idle()
@@ -874,6 +883,12 @@ class AbstractUI(dbus.service.Object):
         '''
         self.ui_init() # we want to show progress
         self.have_ui = True
+        self.search_only = False
+        # Note: This could be set to True if we know that we only look for
+        # remote drivers. E. g. if you don't support local printer driver
+        # handlers, you could do:
+        #if hwid.startswith('printer_deviceid:'):
+        #    self.search_only = True
 
         b = self.backend()
 
@@ -908,6 +923,11 @@ class AbstractUI(dbus.service.Object):
         if self.dbus_server_main_loop:
             self.dbus_server_main_loop.quit()
 
+        # in case we do another operation after that, we need to reinitialize
+        # the backend
+        if self.search_only:
+            self.search_only = False
+            self._dbus_iface = None
         return (result, files)
 
     #
@@ -922,7 +942,7 @@ class AbstractUI(dbus.service.Object):
 
         A double underscore ('__') is converted to a real '_'.'''
 
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_init(self):
         '''Initialize UI.
@@ -930,7 +950,7 @@ class AbstractUI(dbus.service.Object):
         This should load the GUI components, such as GtkBuilder files, but not
         show the main window yet; that is done by ui_show_main().
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
         
     def ui_show_main(self):
         '''Show main window.
@@ -938,7 +958,7 @@ class AbstractUI(dbus.service.Object):
         This should set up presentation of handlers and show the main
         window. This must be called after ui_init().
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_main_loop(self):
         '''Main loop for the user interface.
@@ -946,12 +966,12 @@ class AbstractUI(dbus.service.Object):
         This should return if the user wants to quit the program, and return
         the exit code.
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def error_message(self, title, text):
         '''Present an error message box.'''
 
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def confirm_action(self, title, text, subtext=None, action=None):
         '''Present a confirmation dialog.
@@ -959,7 +979,7 @@ class AbstractUI(dbus.service.Object):
         If action is given, it is used as button label instead of the default
         'OK'.  Return True if the user confirms, False otherwise.
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_notification(self, title, text):
         '''Present a notification popup.
@@ -972,13 +992,13 @@ class AbstractUI(dbus.service.Object):
         python-appindicator) then it should behave as it did before.
         '''
         try:
-            indicator = appindicator.Indicator('jockey', 'jockey',
-                    appindicator.CATEGORY_HARDWARE)
-        except Exception, e:
-            raise NotImplementedError, 'appindicator support not available: %s' \
-                '\nsubclasses need to implement this' % str(e)
+            indicator = AppIndicator.Indicator.new('jockey', 'jockey',
+                    AppIndicator.IndicatorCategory.HARDWARE)
+        except Exception as e:
+            raise NotImplementedError('appindicator support not available: %s' \
+                '\nsubclasses need to implement this' % str(e))
         
-        indicator.set_status(appindicator.STATUS_ATTENTION)
+        indicator.set_status(AppIndicator.IndicatorStatus.ATTENTION)
         return indicator
 
     def ui_idle(self):
@@ -987,12 +1007,12 @@ class AbstractUI(dbus.service.Object):
         This is called while waiting for external processes such as package
         installers.
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_progress_start(self, title, description, total):
         '''Create a progress dialog.'''
 
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_progress_update(self, current, total):
         '''Update status of current progress dialog.
@@ -1003,9 +1023,9 @@ class AbstractUI(dbus.service.Object):
 
         This should return True to cancel, and False otherwise.
         '''
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
 
     def ui_progress_finish(self):
         '''Close the current progress dialog.'''
 
-        raise NotImplementedError, 'subclasses need to implement this'
+        raise NotImplementedError('subclasses need to implement this')
