@@ -88,6 +88,17 @@ class XorgDriverHandler(KernelModuleHandler):
             return _('Reconfiguring X.org video drivers is not '
                 'possible: %s is invalid.') % OSLib.inst.xorg_conf_path
 
+    def available(self):
+        if self.package:
+            cur_abi = OSLib.inst.current_xorg_video_abi()
+            pkg_abi = OSLib.inst.video_driver_abi(self.package)
+            if cur_abi and pkg_abi and cur_abi != pkg_abi:
+                logging.debug('XorgDriverHandler(%s, %s, %s): Disabling as package video ABI %s does not match X.org video ABI %s',
+                        self.module, self.package, self.xorg_driver, pkg_abi, cur_abi)
+                return False
+
+        return KernelModuleHandler.available(self)
+
     def enabled(self):
         if self.xorg_conf:
             if self.xorg_conf.isDriverEnabled(self.xorg_driver) == False:
