@@ -154,9 +154,9 @@ class DbusBackendTest(sandbox.LogTestCase):
         self.assertEqual(self.i_drv.handler_info('kmod:firmwifi')['enabled'], 'True')
         try:
             self.i_drv.set_enabled('kmod:firmwifi', False)
-        except dbus.DBusException, e:
+        except dbus.DBusException as e:
             self.assertEqual(e._dbus_error_name,
-                'org.freedesktop.DBus.Python.ZeroDivisionError')
+                'org.freedesktop.DBus.Python.ZeroDivisionError', 'not a ZeroDivisionError:' + str(e))
             return
         self.fail('set_enabled() did not propagate ZeroDivisionError')
 
@@ -202,7 +202,7 @@ class DbusBackendTest(sandbox.LogTestCase):
 
         try:
             sandbox.start_driverdb_server()
-            self.i_drv.add_driverdb('XMLRPCDriverDB', ['http://localhost:80080'])
+            self.i_drv.add_driverdb('XMLRPCDriverDB', ['http://localhost:8080'])
             self.i_drv.update_driverdb()
             drvs = set(self.i_drv.available('nonfree'))
         finally:
@@ -505,7 +505,7 @@ class LocalBackendTest(sandbox.LogTestCase):
 
         try:
             sandbox.start_driverdb_server()
-            b.add_driverdb('XMLRPCDriverDB', ['http://localhost:80080'])
+            b.add_driverdb('XMLRPCDriverDB', ['http://localhost:8080'])
             #b.update_driverdb()
             drvs = set(b.available())
 
