@@ -25,27 +25,9 @@ import jockey.detection, jockey.backend
 from jockey.detection import HardwareID
 from jockey.oslib import OSLib
 
-import sandbox
+import sandbox, http
 
 class DetectionTest(sandbox.LogTestCase):
-    _has_defaultroute = None
-
-    def has_defaultroute(self):
-        '''Return if there is a default route.
-
-        This is a reasonable indicator that online tests can be run.
-        '''
-        if self._has_defaultroute is None:
-            self._has_defaultroute = False
-            route = subprocess.Popen(['/sbin/route', '-n'],
-                stdout=subprocess.PIPE)
-            for l in route.stdout:
-                if l.startswith('0.0.0.0 '):
-                    self._has_defaultroute = True
-            assert route.wait() == 0
-
-        return self._has_defaultroute
-
     def tearDown(self):
         '''Clean handler dir.'''
 
@@ -308,6 +290,23 @@ kmod:vanilla3d([KernelModuleHandler, nonfree, enabled] nonfree, replacement grap
         for h in result:
             self.assert_(h.enabled())
 
+    def test_localkernelmod_driverdb_disabled_customhandler(self):
+        '''LocalKernelModulesDriverDB properly treats disabled custom handlers
+        
+        It should prefer custom handlers over autogenerated standard ones even
+        if the custom handler is not available, instead of falling back to
+        creating a standard handler.
+        '''
+        open (os.path.join(OSLib.inst.handler_dir, 'testhandlers.py'),
+                'w').write(sandbox.h_notavail_mint_mod)
+        db = jockey.detection.LocalKernelModulesDriverDB()
+
+        result = jockey.detection.get_handlers(jockey.backend.Backend(), db)
+
+        out = '\n'.join(sorted([str(h) for h in result]))
+
+        self.failIf('kmod:mint' in out, out)
+
     def test_localkernelmod_driverdb_overrides(self):
         '''LocalKernelModulesDriverDB respects modalias overrides'''
 
@@ -374,6 +373,31 @@ kmod:vanilla3d([KernelModuleHandler, nonfree, enabled] nonfree, replacement grap
             for f in os.listdir(OSLib.inst.modaliases[1]):
                 os.unlink(os.path.join(OSLib.inst.modaliases[1], f))
 
+    def test_localkernelmod_driverdb_package_headers(self):
+        '''LocalKernelModulesDriverDB reads modaliases from package headers'''
+
+        orig_phm = OSLib.inst.package_header_modaliases
+        try:
+            # assign spam module to the unknown piece of hardware
+            OSLib.inst.package_header_modaliases = lambda: { 
+                    'pretzel': { 'spam': [
+                        'foo:v1234dDEADBEEFx*',
+                        'pci:v0000FFF0d*1sv*sd*bc06sc*i*']
+                    }
+                }
+
+            result = jockey.detection.get_handlers(jockey.backend.Backend(),
+                jockey.detection.LocalKernelModulesDriverDB())
+        finally:
+            OSLib.inst.package_header_modaliases = orig_phm
+
+        for h in result:
+            if h.module == 'spam':
+                self.assertEqual(h.package, 'pretzel')
+                break
+        else:
+            self.fail('no handler for spam module created')
+
     def test_localkernelmod_driverdb_packages(self):
         '''LocalKernelModulesDriverDB properly handles package field'''
 
@@ -425,7 +449,7 @@ class PickyHandler(jockey.handlers.KernelModuleHandler):
         backend = jockey.backend.Backend()
         # should *not* contact server, do that before starting server
         OSLib.inst.set_dmi('', '')
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
 
         # until we update, there is no data
         self.assertEqual(jockey.detection.get_handlers(backend,
@@ -450,7 +474,7 @@ kmod:spam([KernelModuleHandler, free, enabled] free mystical module without moda
         # previous DB should have written a cache
         cache_path = db.cache_path
         self.assert_(os.path.exists(db.cache_path))
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
         result = jockey.detection.get_handlers(backend, db,
             hardware=backend.hardware)
         out = '\n'.join(sorted([str(h) for h in result]))
@@ -467,7 +491,7 @@ kmod:spam([KernelModuleHandler, free, enabled] free mystical module without moda
 
         # should not fall over on damaged cache files
         open(cache_path, 'wb').write('bogus')
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
         self.assertEqual(jockey.detection.get_handlers(backend, db,
             hardware=backend.hardware), set())
 
@@ -479,7 +503,7 @@ kmod:spam([KernelModuleHandler, free, enabled] free mystical module without moda
 
         backend = jockey.backend.Backend()
         # should *not* contact server, do that before starting server
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
 
         # until we update, there is no data
         self.assertEqual(jockey.detection.get_handlers(backend,
@@ -538,7 +562,7 @@ kmod:spam:White_Hat([NoDetectMod, free, enabled] WhiteHat ndmod)''')
         '''XMLRPCDriverDB return localized strings'''
 
         backend = jockey.backend.Backend()
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
         sandbox.start_driverdb_server()
         try:
             db.update(backend.hardware)
@@ -619,7 +643,7 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
 
         # cache writing error
         backend = jockey.backend.Backend()
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
         db.cache_path = '/proc/null'
         sandbox.start_driverdb_server()
         try:
@@ -630,7 +654,7 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         self.assertNotEqual(db.cache, {})
 
         # return protocol mismatch
-        db = jockey.detection.XMLRPCDriverDB('http://localhost:80080')
+        db = jockey.detection.XMLRPCDriverDB('http://localhost:8080')
         sandbox.start_driverdb_server(correct_protocol=False)
         try:
             db.update(backend.hardware)
@@ -663,13 +687,10 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
 
         self.assertEqual(len(h), 3)
 
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
     def test_openprinting_unknownprinter(self):
         '''OpenPrintingDriverDB with an unknown printer'''
 
-        if not self.has_defaultroute():
-            print >> sys.stderr, '[skipped, no default route] ',
-            return
-
         backend = jockey.backend.Backend()
         backend.hardware = set([HardwareID('printer_deviceid',
             'MFG:FooTech;MDL:X-12;CMD:GDI')])
@@ -677,16 +698,13 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         backend.update_driverdb()
         self.assertEqual(backend.available(), [])
 
-    def test_openprinting_apt(self):
-        '''OpenPrintingDriverDB, known printer, apt package system
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
+    def test_openprinting_apt_ppd(self):
+        '''OpenPrintingDriverDB, known printer, apt package system, PPD only
         
         Note that this test case does assumptions about the data returned by
         openprinting.org, thus it needs to be adapted over time.
         '''
-        if not self.has_defaultroute():
-            print >> sys.stderr, '[skipped, no default route] ',
-            return
-
         prn = HardwareID('printer_deviceid', 'MFG:Hewlett-Packard;MDL:HP DesignJet 3500CP')
         db = jockey.detection.OpenPrintingDriverDB()
         orig_pkgsys = OSLib.inst.packaging_system
@@ -710,12 +728,42 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         self.assert_(drv['repository'].startswith('deb '))
         self.assert_('license' in drv)
 
-    def test_openprinting_handler(self):
-        '''OpenPrintingDriverDB: creation of PrinterDriverHandler'''
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
+    def test_openprinting_apt_binary(self):
+        '''OpenPrintingDriverDB, known printer, apt package system, binary
 
-        if not self.has_defaultroute():
-            print >> sys.stderr, '[skipped, no default route] ',
-            return
+        Note that this test case does assumptions about the data returned by
+        openprinting.org, thus it needs to be adapted over time.
+
+        This includes fetching the GPG fingerprint URL and verifying the SSL
+        certificate validity and trust.
+        '''
+        prn = HardwareID('printer_deviceid', 'MFG:EPSON;MDL:Epson ME 320')
+        db = jockey.detection.OpenPrintingDriverDB()
+        orig_pkgsys = OSLib.inst.packaging_system
+        try:
+            OSLib.inst.packaging_system = lambda: 'apt'
+            db.update(set([prn]))
+        finally:
+            OSLib.inst.packaging_system = orig_pkgsys
+
+        drv = db.query(prn)
+        
+        self.assertEqual(len(drv), 1)
+        drv = drv[0].properties
+
+        self.assertEqual(drv['driver_type'], 'printer_driver')
+        self.assert_('Epson' in drv['description']['C'])
+        self.assert_('lineart' in drv['long_description']['C'])
+        self.assert_(drv['package'].startswith('epson-inkjet-'))
+        self.assert_('Epson' in drv['driver_vendor'])
+        self.assert_(drv['repository'].startswith('deb '))
+        self.assertEqual(drv['fingerprint'], 'E522 0FB7 014D 0FBD A50D  FC2B E5E8 6C00 8AA6 5D56') 
+        self.assert_('license' in drv)
+
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
+    def test_openprinting_handler_ppd(self):
+        '''OpenPrintingDriverDB: PrinterDriverHandler for PPD package'''
 
         backend = jockey.backend.Backend()
         backend.hardware = set([HardwareID('printer_deviceid',
@@ -733,20 +781,39 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         hi = backend.handler_info(handlers[0])
         self.assertEqual(hi['package'], 'openprinting-ppds-postscript-hp')
         self.assertEqual(hi['free'], 'True')
-        self.assert_('package' in hi)
         self.assert_('repository' in hi)
         self.assert_('driver_vendor' in hi)
 
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
+    def test_openprinting_handler_binary(self):
+        '''OpenPrintingDriverDB: PrinterDriverHandler for binary package'''
+
+        backend = jockey.backend.Backend()
+        backend.hardware = set([HardwareID('printer_deviceid',
+            'MFG:EPSON;MDL:Epson ME 320')])
+        backend.driver_dbs.append(jockey.detection.OpenPrintingDriverDB())
+        orig_pkgsys = OSLib.inst.packaging_system
+        try:
+            OSLib.inst.packaging_system = lambda: 'apt'
+            backend.update_driverdb()
+        finally:
+            OSLib.inst.packaging_system = orig_pkgsys
+        handlers = backend.available()
+        self.assertEqual(handlers, ['printer:epson-inkjet-printer-n10-nx127:epson-n10-nx127'])
+
+        hi = backend.handler_info(handlers[0])
+        self.assert_(hi['package'].startswith('epson-inkjet-'))
+        self.assert_(hi['repository'].startswith('deb '))
+        self.assert_('repository_sign_fp' in hi)
+        self.assert_('driver_vendor' in hi)
+
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
     def test_openprinting_yum(self):
         '''OpenPrintingDriverDB, known printer, yum package system
         
         Note that this test case does assumptions about the data returned by
         openprinting.org, thus it needs to be adapted over time.
         '''
-        if not self.has_defaultroute():
-            print >> sys.stderr, '[skipped, no default route] ',
-            return
-
         prn = HardwareID('printer_deviceid', 'MFG:Hewlett-Packard;MDL:HP DesignJet 3500CP')
         db = jockey.detection.OpenPrintingDriverDB()
 
@@ -768,16 +835,13 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
         self.assertEqual(drv['package'], 'openprinting-ppds-postscript-hp')
         self.assertEqual(drv['driver_vendor'], 'HP')
         self.assertEqual(drv['free'], True)
-        self.assert_('printdriver/rpms' in drv['repository'].lower())
+        self.assert_('rpms' in drv['repository'].lower())
         self.assert_('license' in drv)
 
+    @unittest.skipUnless(OSLib.has_defaultroute(), 'online test')
     def test_openprinting_unknownpkg(self):
         '''OpenPrintingDriverDB, known printer, unknown package system'''
 
-        if not self.has_defaultroute():
-            print >> sys.stderr, '[skipped, no default route] ',
-            return
-
         prn = HardwareID('printer_deviceid', 'MFG:Hewlett-Packard;MDL:HP DesignJet 3500CP')
         db = jockey.detection.OpenPrintingDriverDB()
         orig_pkgsys = OSLib.inst.packaging_system
@@ -789,3 +853,87 @@ kmod:vanilla3d([VanillaGfxHandler, nonfree, enabled] nonfree, replacement graphi
 
         drv = db.query(prn)
         self.assertEqual(len(drv), 0)
+
+class GPGFingerPrintTest(sandbox.LogTestCase):
+    def setUp(self):
+        self.data_dir = os.path.join(os.path.dirname(__file__), 'data')
+        self.localhost_pem = os.path.join(self.data_dir, 'localhost.pem')
+        self.localhost_expired_pem = os.path.join(self.data_dir, 'localhost-expired.pem')
+        self.orig_cert_file_paths = OSLib.inst.ssl_cert_file_paths
+        OSLib.inst.ssl_cert_file_paths = [self.localhost_pem]
+
+    def tearDown(self):
+        http.stop()
+        OSLib.inst.ssl_cert_file_paths = self.orig_cert_file_paths
+
+    def test_valid(self):
+        '''GPG fingerprint: valid certificate, hostname, and fingerprint'''
+
+        http.start(4443, self.data_dir, True, self.localhost_pem)
+
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://localhost:4443/fingerprint')
+        self.assertEqual(result, '0000 1111 2222 3333 4444  5555 6666 7777 8888 9999')
+
+    def test_bad_fingerprint_syntax(self):
+        '''GPG fingerprint: no fingerprint found'''
+
+        http.start(4443, self.data_dir, True, self.localhost_pem)
+
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://localhost:4443/bad_fingerprint')
+        self.assertEqual(result, None, result)
+
+    def test_404(self):
+        '''GPG fingerprint: nonexisting address'''
+
+        http.start(4443, self.data_dir, True, self.localhost_pem)
+
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://localhost:4443/nonexisting')
+        self.assertEqual(result, None, result)
+
+    def test_wrong_host(self):
+        '''GPG fingerprint: unknown hostname'''
+
+        http.start(4443, self.data_dir, True, self.localhost_pem)
+
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://127.0.0.1:4443/fingerprint')
+        self.assertEqual(result, None, result)
+
+    def test_expired_cert(self):
+        '''expired certificate'''
+
+        http.start(4443, self.data_dir, True, self.localhost_expired_pem)
+
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://127.0.0.1:4443/fingerprint')
+        self.assertEqual(result, None, result)
+
+    def test_wrong_cert(self):
+        '''wrong certificate'''
+
+        OSLib.inst.ssl_cert_file_paths = [self.localhost_expired_pem]
+        http.start(4443, '.', True, self.localhost_pem)
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://127.0.0.1:4443/fingerprint')
+        self.assertEqual(result, None, result)
+
+    def test_system_cert(self):
+        '''system-wide trusted certificate shouldn't match local test server'''
+
+        if not self.orig_cert_file_paths:
+            print '[SKIP, not available]'
+            return
+
+        OSLib.inst.ssl_cert_file_paths = self.orig_cert_file_paths
+        http.start(4443, '.', True, self.localhost_pem)
+
+        result = jockey.detection.download_gpg_fingerprint(
+                'https://127.0.0.1:4443/fingerprint')
+        self.assertEqual(result, None, result)
+
+if __name__ == '__main__':
+    OSLib.inst = sandbox.TestOSLib()
+    unittest.main()
