@@ -903,6 +903,28 @@ EndSection
         self.failIf(h.changed())
         self.failIf(h.enabled())
 
+    def test_xorg_driver_abi_checking(self):
+        '''XorgDriverHandler ABI checking'''
+
+        h = XorgDriverHandler(self.backend, 'vanilla3d', 'mesa-vanilla', 'v3d',
+            'vanilla', name='Vanilla accelerated graphics driver')
+
+        # our TestOSLib doesn't define video_driver_abi(), so should be
+        # available by default
+        self.assertNotEqual(h.available(), False)
+
+        orig_video_driver_abi = OSLib.inst.video_driver_abi
+        try:
+            # should be available for matching ABI
+            OSLib.inst.video_driver_abi = lambda pkg: OSLib.inst.current_xorg_video_abi()
+            self.assertNotEqual(h.available(), False)
+
+            # should not be available for nonmatching ABI
+            OSLib.inst.video_driver_abi = lambda pkg: 'otherabi'
+            self.assertEqual(h.available(), False)
+        finally:
+            OSLib.inst.video_driver_abi = orig_video_driver_abi
+
     def test_firmware_handler(self):
         '''standard FirmwareHandler'''
 
