@@ -175,6 +175,7 @@ fake_db = {
             ('Foonux', '42'): [
                 DriverID(jockey_handler='KernelModuleHandler', kernel_module='chocolate',
                     repository='http://vaportech.com/x1',
+                    fingerprint='0000 FFFF 1111 EEEE 2222  DDDD 3333 CCCC 4444 BBBB',
                     package='spam-x1', driver_vendor='VaporTech',
                     description={'C': 'VaporTech choc handler', 'de_DE':
                         'German description', 'cs': 'Czech description'},
@@ -371,7 +372,7 @@ else:
 
 ''' % repr(fake_modinfo))
         mi.close()
-        os.chmod(self.modinfo_path, 0755)
+        os.chmod(self.modinfo_path, 0o755)
 
     def _make_modprobe(self):
         '''Create a dummy modprobe and set self.modprobe_path.'''
@@ -398,7 +399,7 @@ else:
 ''' % (repr(fake_modinfo), repr(self.proc_modules)))
 
         mp.close()
-        os.chmod(self.modprobe_path, 0755)
+        os.chmod(self.modprobe_path, 0o755)
         
     def _make_sys(self):
         '''Create a dummy /sys tree from fake_sys and set self.sys_dir.'''
@@ -549,7 +550,7 @@ EndSection
                 return fake_pkginfo[repo][package]['description']
             except KeyError:
                 continue
-        raise ValueError, 'no such package'
+        raise ValueError('no such package')
 
     def is_package_free(self, package):
         '''Return if given package is free software.'''
@@ -559,7 +560,7 @@ EndSection
                 return fake_pkginfo[repo][package]['free']
             except KeyError:
                 continue
-        raise ValueError, 'no such package'
+        raise ValueError('no such package')
 
     def package_installed(self, package):
         '''Return if the given package is installed.'''
@@ -577,9 +578,10 @@ EndSection
                     return fake_pkginfo[repo][package]['files']
                 except KeyError:
                     continue
-        raise ValueError, 'package not installed'
+        raise ValueError('package not installed')
 
-    def install_package(self, package, progress_cb):
+    def install_package(self, package, progress_cb, repository=None,
+            fingerprint=None):
         '''Install the given package.
 
         As this is called in the backend, this must happen noninteractively.
@@ -592,18 +594,36 @@ EndSection
         If this succeeds, subsequent package_installed(package) calls must
         return True.
 
-        Any installation failure should be raised as a SystemError.
+        If a repository URL is given, that repository is added to the system
+        first. The format for repository is distribution specific. This function
+        should also download/update the package index for this repository when
+        adding it.
+        .
+        fingerprint, if not None, is a GPG-style fingerprint of that
+        repository; if present, this method should also retrieve that GPG key
+        from the keyservers, install it into the packaging system, and ensure
+        that the repository is signed with that key.
+
+        An unknown package should raise a ValueError. Any installation failure
+        should be raised as a SystemError.
         '''
+        if repository:
+            if repository not in fake_pkginfo:
+                raise SystemError('repository %s does not exist' % repository)
+            if fingerprint and not fingerprint.startswith('0000 FFFF'):
+                raise SystemError('invalid repository fingerprint')
+            self.repositories.add(repository)
+
         for repo in self.repositories:
             if package in fake_pkginfo[repo]:
                 break
         else:
-            raise ValueError, 'no such package'
+            raise ValueError('no such package')
 
         if self.pending_install_remove_exception:
             e = self.pending_install_remove_exception
             self.pending_install_remove_exception = None
-            raise SystemError, e
+            raise SystemError(e)
 
         if self.next_package_install_fails:
             self.next_package_install_fails = False
@@ -636,7 +656,7 @@ EndSection
         if self.pending_install_remove_exception:
             e = self.pending_install_remove_exception
             self.pending_install_remove_exception = None
-            raise SystemError, e
+            raise SystemError(e)
 
         self.installed_packages.remove(package)
 
@@ -646,32 +666,6 @@ EndSection
             time.sleep(0.2)
             progress_cb(2, 2)
 
-    def add_repository(self, repository):
-        '''Add a repository.
-
-        The format for repository is distribution specific. This function
-        should also download/update the package index for this repository.
-
-        This should throw a ValueError if the repository is invalid or
-        inaccessible.
-        '''
-
-        if repository not in fake_pkginfo:
-            raise ValueError, 'repository %s does not exist' % repository
-
-        self.repositories.add(repository)
-
-    def remove_repository(self, repository):
-        '''Remove a repository.
-
-        The format for repository is distribution specific.
-        '''
-
-        try:
-            self.repositories.remove(repository)
-        except KeyError:
-            pass
-
     def repository_enabled(self, repository):
         '''Check if given repository is enabled.'''
 
@@ -694,12 +688,20 @@ EndSection
         '''
         self.help_called = True
 
+    def current_xorg_video_abi(self):
+        '''Return current X.org video ABI.
+        
+        This returns a static string for the test suite. Tests are done by
+        dynamically implementing video_driver_abi().
+        '''
+        return 'X.org-video-1test'
+
 #-------------------------------------------------------------------#
 
 class AllAvailOSLib(OSLib):
     '''Test suite implementation of OSLib.
 
-    This pretends that any module and packge is always available and gives
+    This pretends that any module and package is always available and gives
     predictable and constant descriptions and states. It is mainly useful for
     testing arbitrary and unknown handlers.
     '''
@@ -739,7 +741,8 @@ class AllAvailOSLib(OSLib):
 
         return package in self.installed_packages
 
-    def install_package(self, package, progress_cb):
+    def install_package(self, package, progress_cb, repository=None,
+            fingerprint=None):
         '''Install the given package.'''
 
         self.installed_packages.add(package)
@@ -794,7 +797,7 @@ print '%%-16s%%s' %% ('license:', 'GPL')
 print '%%-16s%%s' %% ('description:', '%%s module' %% m)
 ''' % repr(fake_modinfo))
         mi.close()
-        os.chmod(self.modinfo_path, 0755)
+        os.chmod(self.modinfo_path, 0o755)
 
 #-------------------------------------------------------------------#
 
@@ -981,7 +984,7 @@ def xmlrpc_driverdb_query_bogus(proto_ver, proto_subver, request):
 def start_driverdb_server(correct_protocol=True):
     '''Create XML-RPC driver db server.
 
-    This will listen on localhost:80080 and serve fake_db. This must be
+    This will listen on localhost:8080 and serve fake_db. This must be
     terminated with stop_driverdb_server().
 
     If correct_protocol is False, this will answer with a bogus protocol
@@ -991,7 +994,7 @@ def start_driverdb_server(correct_protocol=True):
     start_driverdb_server.pid = os.fork()
     if start_driverdb_server.pid == 0:
         try:
-            s = SimpleXMLRPCServer(('localhost', 80080), logRequests=False)
+            s = SimpleXMLRPCServer(('localhost', 8080), logRequests=False)
             s.register_introspection_functions()
             if correct_protocol:
                 s.register_function(xmlrpc_driverdb_query, 'query')
@@ -1004,7 +1007,7 @@ def start_driverdb_server(correct_protocol=True):
             os._exit(1)
 
     # wait until it is ready
-    c = xmlrpclib.ServerProxy('http://localhost:80080')
+    c = xmlrpclib.ServerProxy('http://localhost:8080')
     timeout = 100
     while timeout > 0:
         time.sleep(0.1)
@@ -1015,7 +1018,7 @@ def start_driverdb_server(correct_protocol=True):
             continue
         break
     if timeout == 0:
-        raise SystemError, 'XML-RPC demo server did not start'
+        raise SystemError('XML-RPC demo server did not start')
 
 def stop_driverdb_server():
     '''Stop the XML-RPC driver db server.'''
@@ -1157,15 +1160,18 @@ alias pci:v00001001d00003D*sv*sd*bc03sc*i* vanilla3d
         self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
         self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), False)
 
-        OSLib.inst.add_repository('http://vaportech.com/x1')
+        # wrong fingerprint
+        self.assertRaises(SystemError, OSLib.inst.install_package, 'foo-y1', None,
+                'http://vaportech.com/x1', '0xDEADBEEF')
+        self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), False)
+
+        OSLib.inst.install_package('foo-y1', None, 'http://vaportech.com/x1', None)
         self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), True)
 
         self.assertEqual(OSLib.inst.package_description('foo-y1'),
             fake_pkginfo['http://vaportech.com/x1']['foo-y1']['description'])
-        self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
-        self.assertEqual(OSLib.inst.is_package_free('foo-y1'), True)
-        OSLib.inst.install_package('foo-y1', None)
         self.assertEqual(OSLib.inst.package_installed('foo-y1'), True)
+        self.assertEqual(OSLib.inst.is_package_free('foo-y1'), True)
 
         # test exception raising
         OSLib.inst.pending_install_remove_exception = 'moo!'
@@ -1175,16 +1181,11 @@ alias pci:v00001001d00003D*sv*sd*bc03sc*i* vanilla3d
         self.assertRaises(SystemError, OSLib.inst.remove_package, 'foo-y1', None)
         self.assertEqual(OSLib.inst.package_installed('foo-y1'), True)
 
-        OSLib.inst.remove_repository('foo')
-
-        OSLib.inst.remove_repository('http://vaportech.com/x1')
-        self.assertEqual(OSLib.inst.repository_enabled('http://vaportech.com/x1'), False)
         self.assertEqual(OSLib.inst.package_installed('foo-y1'), True)
 
         OSLib.inst.remove_package('foo-y1', None)
         self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
 
-        self.assertRaises(ValueError, OSLib.inst.package_description, 'foo-y1')
         self.assertEqual(OSLib.inst.package_installed('foo-y1'), False)
         self.assertEqual(OSLib.inst.package_description('mesa-vanilla'),
             fake_pkginfo['default']['mesa-vanilla']['description'])
@@ -1197,7 +1198,7 @@ alias pci:v00001001d00003D*sv*sd*bc03sc*i* vanilla3d
 class TestXMLRPCDriverDBConsistency(unittest.TestCase):
     def setUp(self):
         start_driverdb_server()
-        self.xmlrpc_client = xmlrpclib.ServerProxy('http://localhost:80080')
+        self.xmlrpc_client = xmlrpclib.ServerProxy('http://localhost:8080')
 
     def tearDown(self):
         stop_driverdb_server()
@@ -1255,7 +1256,8 @@ class TestXMLRPCDriverDBConsistency(unittest.TestCase):
                          'German description', 'cs': 'Czech description'},
                      'long_description': {'C': 'Multi\nline'},
                      'driver_vendor': 'VaporTech', 'package': 'spam-x1', 'repository':
-                     'http://vaportech.com/x1', 'version': '1.1-foo42-vt3'},
+                     'http://vaportech.com/x1', 'version': '1.1-foo42-vt3',
+                     'fingerprint': '0000 FFFF 1111 EEEE 2222  DDDD 3333 CCCC 4444 BBBB'},
                     {'driver_type': 'kernel_module', 'jockey_handler':
                      'NoDetectMod', 'driver_vendor': 'White Hat',
                      'description': {'C': 'WhiteHat ndmod'}},
@@ -1319,6 +1321,16 @@ class NotAvailMod(KernelModuleHandler):
         return False
 '''
 
+h_notavail_mint_mod = '''
+from jockey.handlers import KernelModuleHandler
+class NotAvailMod(KernelModuleHandler):
+    def __init__(self, ui):
+        KernelModuleHandler.__init__(self, ui, 'mint',
+            description='Test suite: non-available kernel module')
+    def available(self):
+        return False
+'''
+
 h_nodetectmod = '''
 class NoDetectMod(jockey.handlers.KernelModuleHandler):
     def __init__(self, ui):
