@@ -18,15 +18,14 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-import unittest, sys, os, signal, time, traceback, urllib, tempfile
+import unittest, sys, os, signal, time, traceback, tempfile
 from cStringIO import StringIO
-import SimpleHTTPServer, BaseHTTPServer
 
 import jockey.ui, jockey.handlers
 from jockey.oslib import OSLib
 from jockey.xorg_driver import XorgDriverHandler
 
-import sandbox
+import sandbox, http
 
 class UITest(sandbox.LogTestCase):
     '''User interface tests.
@@ -594,57 +593,17 @@ class UITest(sandbox.LogTestCase):
         self.assert_(ratio >= 0.3, ratio)
         self.assert_(ratio < 0.5)
 
-    def _fork_http_server(self):
-        '''fork a HTTP server on OSLib workdir which serves one request and
-        terminates.'''
-
-        # fork test HTTP server
-        pid = os.fork()
-        if pid == 0:
-            try:
-                os.chdir(OSLib.inst.workdir)
-                httpd = BaseHTTPServer.HTTPServer(('', 8427),
-                    SimpleHTTPServer.SimpleHTTPRequestHandler)
-                # first request is for startup check, second for the actual test
-                httpd.handle_request()
-                httpd.handle_request()
-                os._exit(0)
-            except:
-                print '********** UI HTTP server failed: ***********'
-                traceback.print_exc()
-                os._exit(1)
-        else:
-            self.http_pid = pid
-            # wait until server is ready
-            while True:
-                time.sleep(0.1)
-                try:
-                    f = urllib.urlopen('http://localhost:8427')
-                except IOError:
-                    continue
-                f.read()
-                f.close()
-                break
-
-    def _join_http_server(self, kill=False):
-        '''Wait for the forked HTTP server to finish and assert that it exits
-        with zero.'''
-
-        if kill:
-            os.kill(self.http_pid, signal.SIGTERM)
-            os.wait() # undefined exit code
-        else:
-            self.assertEqual(os.wait(), (self.http_pid, 0))
-
     def test_download_http_nocancel(self):
         '''download_url(), HTTP, no cancelling'''
 
+        self.stop_capture()
+
         ui = sandbox.TestUI()
 
         # temporary file
-        self._fork_http_server()
+        http.start(8427, OSLib.inst.workdir)
         (fname, h) = ui.download_url('http://localhost:8427/stuff')
-        self._join_http_server()
+        http.stop()
         self.assertEqual(open(fname).read(), self.big_file_contents)
         os.unlink(fname)
 
@@ -656,9 +615,9 @@ class UITest(sandbox.LogTestCase):
 
         # specified file name
         dest = os.path.join(OSLib.inst.workdir, 'destfile')
-        self._fork_http_server()
+        http.start(8427, OSLib.inst.workdir)
         (fname, h) = ui.download_url('http://localhost:8427/stuff', dest)
-        self._join_http_server()
+        http.stop()
         self.assertEqual(fname, dest)
         self.assertEqual(open(dest).read(), self.big_file_contents)
         os.unlink(dest)
@@ -670,9 +629,9 @@ class UITest(sandbox.LogTestCase):
             len(self.big_file_contents), len(self.big_file_contents)])
 
         # nonexisting file
-        self._fork_http_server()
+        http.start(8427, OSLib.inst.workdir)
         (fname, h) = ui.download_url('http://localhost:8427/nonexisting')
-        self._join_http_server()
+        http.stop()
         self.assertEqual(fname, None)
 
         # one error message about download failure
@@ -680,9 +639,9 @@ class UITest(sandbox.LogTestCase):
         self.assertRaises(IndexError, ui.pop_error)
 
         # nonexisting server
-        self._fork_http_server()
-        (fname, h) = ui.download_url('http://i.do.not.exist/nonexisting')
-        self._join_http_server(kill=True)
+        http.start(8427, OSLib.inst.workdir)
+        (fname, h) = ui.download_url('http://i.do.not.exist:8080/nonexisting')
+        http.stop()
         self.assertEqual(fname, None)
 
         # one error message about download failure
@@ -692,13 +651,15 @@ class UITest(sandbox.LogTestCase):
     def test_download_http_cancel(self):
         '''download_url(), HTTP, cancelling'''
 
+        self.stop_capture()
+
         ui = sandbox.TestUI()
 
         # temporary file
-        self._fork_http_server()
+        http.start(8427, OSLib.inst.workdir)
         ui.cancel_progress = True
         (fname, h) = ui.download_url('http://localhost:8427/stuff')
-        self._join_http_server()
+        http.stop()
         self.assertEqual(fname, None)
 
         self.assertRaises(IndexError, ui.pop_error)
@@ -712,9 +673,9 @@ class UITest(sandbox.LogTestCase):
         # specified file name
         dest = os.path.join(OSLib.inst.workdir, 'destfile')
         ui.cancel_progress = True
-        self._fork_http_server()
+        http.start(8427, OSLib.inst.workdir)
         (fname, h) = ui.download_url('http://localhost:8427/stuff', dest)
-        self._join_http_server()
+        http.stop()
         self.assertEqual(fname, None)
 
         self.assertRaises(IndexError, ui.pop_error)
@@ -813,7 +774,7 @@ class UITest(sandbox.LogTestCase):
         try:
             iface.search_driver('unknown:bar')
             fail('should only accept one method call and then terminate')
-        except Exception, e:
+        except Exception as e:
             self.assertEqual(e._dbus_error_name, 'org.freedesktop.DBus.Error.NoReply')
 
         # give the daemon a second to terminate
@@ -858,7 +819,7 @@ class UITest(sandbox.LogTestCase):
 
         sandbox.start_driverdb_server()
         try:
-            ui.backend().add_driverdb('XMLRPCDriverDB', ['http://localhost:80080'])
+            ui.backend().add_driverdb('XMLRPCDriverDB', ['http://localhost:8080'])
             # first match is kmod:mint:UbuOne in our fake db
             result = ui.search_driver('pci:9876/FEDC')
         finally:
